/* Special definitions for GNU plotutils, processed by autoheader.
   Copyright (C) 1997 Free Software Foundation, Inc.
   Robert S. Maier <rsm@math.arizona.edu>, 1997.
*/

/* Define to 1 under HP/UX to get matherr-related things from math.h. */
#undef _HPUX_SOURCE
